"use strict";
var factory_1 = require("../core/factory");
var registrar_1 = require("../router/registrar");
var enums_1 = require("../router/enums");
var metadata_1 = require("../router/metadata");
function Handler(path) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        if (args.length === 1) {
            return HandlerDecorator.apply(this, [args[0], path]);
        }
        throw new Error("Invalid @Handler Decorator declaration.");
    };
}
exports.Handler = Handler;
function HandlerDecorator(target, path) {
    var registrar = factory_1.Container.get(registrar_1.RouteRegistrar);
    var routeHandler = registrar.addHttpHandler(target);
    if (routeHandler) {
        routeHandler.path = path;
    }
}
function Route(path) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        if (args.length === 1) {
            return RouteAreaDecorator.apply(this, [args[0], path]);
        }
        else if (args.length === 3 && typeof args[2] === "object") {
            return RouteActionDecorator.apply(this, [args[0], args[1], args[2], path]);
        }
        throw new Error("Invalid @Route Decorator declaration.");
    };
}
exports.Route = Route;
function RouteAreaDecorator(target, path) {
    var registrar = factory_1.Container.get(registrar_1.RouteRegistrar);
    var routeArea = registrar.addRouteArea(target);
    routeArea.path = path;
}
function RouteActionDecorator(target, propertyKey, descriptor, path) {
    var registrar = factory_1.Container.get(registrar_1.RouteRegistrar);
    var routeHandler = registrar.addRouteHandler(target, propertyKey);
    if (routeHandler) {
        routeHandler.path = path;
        if (!routeHandler.httpVerb) {
            routeHandler.httpVerb = enums_1.HttpVerb.GET;
        }
        routeHandler.name = propertyKey;
    }
}
function Param(name) {
    return function (target, propertyKey, parameterIndex) {
        processDecoratedParameter(target, propertyKey, parameterIndex, enums_1.ParamType.path, name);
    };
}
exports.Param = Param;
function processDecoratedParameter(target, propertyKey, parameterIndex, paramType, name) {
}
function GET(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.GET);
}
exports.GET = GET;
function POST(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.POST);
}
exports.POST = POST;
function PUT(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.PUT);
}
exports.PUT = PUT;
function DELETE(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.DELETE);
}
exports.DELETE = DELETE;
function HEAD(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.HEAD);
}
exports.HEAD = HEAD;
function OPTIONS(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.OPTIONS);
}
exports.OPTIONS = OPTIONS;
function PATCH(target, propertyKey, descriptor) {
    processHttpVerb(target, propertyKey, enums_1.HttpVerb.PATCH);
}
exports.PATCH = PATCH;
function processHttpVerb(target, propertyKey, httpVerb) {
    var registrar = factory_1.Container.get(registrar_1.RouteRegistrar);
    var routeHandler = registrar.addRouteHandler(target, propertyKey);
    if (routeHandler) {
        if (routeHandler.httpVerb) {
            throw new Error("Method is already annotated with @" +
                routeHandler.httpVerb +
                ". You can only map a method to one HTTP verb.");
        }
        routeHandler.httpVerb = httpVerb;
        processRouteHandler(target, propertyKey, routeHandler);
    }
}
function processRouteHandler(target, propertyKey, routeHandler) {
    routeHandler.name = propertyKey;
    var paramTypes = Reflect.getOwnMetadata("design:paramtypes", target, propertyKey);
    while (paramTypes.length > routeHandler.parameters.length) {
        routeHandler.parameters.push(new metadata_1.MethodParam(null, paramTypes[routeHandler.parameters.length], enums_1.ParamType.body));
    }
    routeHandler.parameters.forEach(function (param) {
        if (param.paramType === enums_1.ParamType.cookie) {
            routeHandler.mustParseCookies = true;
        }
        else if (param.paramType === enums_1.ParamType.file) {
            routeHandler.files.push(new metadata_1.FileParam(param.name, true));
        }
        else if (param.paramType === enums_1.ParamType.files) {
            routeHandler.files.push(new metadata_1.FileParam(param.name, false));
        }
        else if (param.paramType === enums_1.ParamType.form) {
            if (routeHandler.mustParseBody) {
                throw Error("Can not use form parameters with a body parameter on the same method.");
            }
            routeHandler.mustParseForms = true;
        }
        else if (param.paramType === enums_1.ParamType.body) {
            if (routeHandler.mustParseForms) {
                throw Error("Can not use form parameters with a body parameter on the same method.");
            }
            if (routeHandler.mustParseBody) {
                throw Error("Can not use more than one body parameter on the same method.");
            }
            routeHandler.mustParseBody = true;
        }
    });
}

//# sourceMappingURL=../../../maps/src/system/router/decorators.js.map
